﻿using NadekoBot.Services.Database.Models;
using System.Linq;
using Microsoft.EntityFrameworkCore;
using System;

namespace NadekoBot.Services.Database.Repositories.Impl
{
    public class ClubRepository : Repository<ClubInfo>, IClubRepository
    {
        public ClubRepository(DbContext context) : base(context)
        {
        }

        public ClubInfo GetByOwner(ulong userId, Func<DbSet<ClubInfo>, IQueryable<ClubInfo>> func = null)
        {
            if (func == null)
                return _set
                    .Include(x => x.Bans)
                    .Include(x => x.Applicants)
                    .Include(x => x.Users)
                    .Include(x => x.Owner)
                    .FirstOrDefault(x => x.Owner.UserId == userId);

            return func(_set).FirstOrDefault(x => x.Owner.UserId == userId);
        }

        public ClubInfo GetByName(string name, int discrim, Func<DbSet<ClubInfo>, IQueryable<ClubInfo>> func = null)
        {
            if (func == null)
                return _set
                    .Include(x => x.Bans)
                    .Include(x => x.Applicants)
                    .Include(x => x.Users)
                    .FirstOrDefault(x => x.Name.ToLowerInvariant() == name && x.Discrim == discrim);

            return func(_set).FirstOrDefault(x => x.Name == name && x.Discrim == discrim);
        }

        public int GetNextDiscrim(string clubName)
        {
            return _set
                .Where(x => x.Name.ToLowerInvariant() == clubName.ToLowerInvariant())
                .Select(x => x.Discrim)
                .DefaultIfEmpty()
                .Max() + 1;
        }

        public ClubInfo GetByMember(ulong userId, Func<DbSet<ClubInfo>, IQueryable<ClubInfo>> func = null)
        {
            if (func == null)
                return _set.Include(x => x.Users)
                    .Include(x => x.Bans)
                    .Include(x => x.Applicants)
                    .FirstOrDefault(x => x.Users.Any(y => y.UserId == userId));

            return func(_set).FirstOrDefault(x => x.Users.Any(y => y.UserId == userId));
        }

        public ClubInfo[] GetClubLeaderboardPage(int page)
        {
            return _set
                .OrderByDescending(x => x.Xp)
                .Skip(page * 9)
                .Take(9)
                .ToArray();
        }
    }
}
